==> alu-json-diff-0.0.3.tgz: package/lib/diff.js <==
const getEditedPaths = (oldStructPaths, newStructPaths) => {
  const diffs = [];
  for (const key in oldStructPaths) {
    if (newStructPaths.hasOwnProperty(key)) {
      if (
        typeof oldStructPaths[key] === "object" &&
        typeof newStructPaths[key] === "object" &&
        JSON.stringify(oldStructPaths[key]) ===
          JSON.stringify(newStructPaths[key])
      ) {
        continue;
      }
      if (oldStructPaths[key] !== newStructPaths[key]) {
        diffs.push([key, oldStructPaths[key], newStructPaths[key]]);
      }
    }
  }
  return diffs;
};

const getPathsDiff = (oldStructPaths, newStructPaths) => {
  const diff = [];
  let index = 0;
  for (const key in oldStructPaths) {
    if (!(key in newStructPaths)) {
      diff[index] = [key, oldStructPaths[key]];
      index++;
    }
  }
  return diff;
};

/**
 * @description 生成 key path 的结构
 */
const generatePath = (currentPath, newPath) => {
  const prefix = ".";
  const path =
    currentPath !== "" ? `${currentPath}${prefix}${newPath}` : `${newPath}`;
  return path;
};

/**
 * @description 获取结构体的 key path, 用作差异性比对
 * 
 * @usageNotes
 * ```
 * const oldStruct = { a: { a1: 1, a2: { a22: 2 } } };
   const result = getStructPaths(oldStruct);
   console.log(result); // { 'a.a1': 1, 'a.a2.a22': 2 }
 * ```
 */
const getStructPaths = (struct, paths = {}, currentPath = "") => {
  for (const key of Object.keys(struct)) {
    const path = generatePath(currentPath, key);
    if (typeof struct[key] === "object" && struct[key] !== null) {
      if (Object.keys(struct[key]).length === 0) {
        paths[path] = struct[key];
      }
      getStructPaths(struct[key], paths, path);
    } else {
      paths[path] = struct[key];
    }
  }
  return paths;
};

/**
 * @description 对比两个 Json 数据
 * 
 * @usageNotes
 * ```
  const oldJson = { a: "a1", b: "b1", c: { c1: "c1" } };
  const newJson = { a: "a1", c: { c1: "c222" }, d: "d1" };

  const result = diff(oldJson, newJson);
  console.log(result); 
  // {
  // added: [ [ 'd', 'd1' ] ],
  // removed: [ [ 'b', 'b1' ] ],
  // edited: [ [ 'c.c1', 'c1', 'c222' ] ]
  // }
 * 
 * ```
 */
export const diff = (oldStruct, newStruct) => {
  const delta = {
    added: [],
    removed: [],
    edited: [],
  };
  const oldStructPaths = getStructPaths(oldStruct);
  const newStructPaths = getStructPaths(newStruct);

  delta.removed = getPathsDiff(oldStructPaths, newStructPaths);
  delta.added = getPathsDiff(newStructPaths, oldStructPaths);
  delta.edited = getEditedPaths(oldStructPaths, newStructPaths);

  return delta;
};

==> alu-json-diff-0.0.8.tgz: package/lib/diff.js <==
const getEditedPaths = (oldStructPaths, newStructPaths) => {
  const diffs = [];
  for (const key in oldStructPaths) {
    if (newStructPaths.hasOwnProperty(key)) {
      if (
        typeof oldStructPaths[key] === "object" &&
        typeof newStructPaths[key] === "object" &&
        JSON.stringify(oldStructPaths[key]) ===
          JSON.stringify(newStructPaths[key])
      ) {
        continue;
      }
      if (oldStructPaths[key] !== newStructPaths[key]) {
        diffs.push([key, oldStructPaths[key], newStructPaths[key]]);
      }
    }
  }
  return diffs;
};

const getPathsDiff = (oldStructPaths, newStructPaths) => {
  const diff = [];
  let index = 0;
  for (const key in oldStructPaths) {
    if (!(key in newStructPaths)) {
      diff[index] = [key, oldStructPaths[key]];
      index++;
    }
  }
  return diff;
};

/**
 * @description 生成 key path 的结构
 */
const generatePath = (currentPath, newPath) => {
  // 可能存在 {'key.key1': "value"} 的结构，所以使用 / 作为分隔符
  const prefix = "/";
  const path =
    currentPath !== "" ? `${currentPath}${prefix}${newPath}` : `${newPath}`;
  return path;
};

/**
 * @description 获取结构体的 key path, 用作差异性比对
 * 
 * @usageNotes
 * ```
 * const oldStruct = { a: { a1: 1, a2: { a22: 2 } } };
   const result = getStructPaths(oldStruct);
   console.log(result); // { 'a/a1': 1, 'a/a2/a22': 2 }
 * ```
 */
const getStructPaths = (struct, paths = {}, currentPath = "") => {
  for (const key of Object.keys(struct)) {
    const path = generatePath(currentPath, key);
    if (typeof struct[key] === "object" && struct[key] !== null) {
      if (Object.keys(struct[key]).length === 0) {
        paths[path] = struct[key];
      }
      getStructPaths(struct[key], paths, path);
    } else {
      paths[path] = struct[key];
    }
  }
  return paths;
};

/**
 * @description 对比两个 Json 数据
 * 
 * @usageNotes
 * ```
  const oldJson = { a: "a1", b: "b1", c: { c1: "c1" } };
  const newJson = { a: "a1", c: { c1: "c222" }, d: "d1" };

  const result = diff(oldJson, newJson);
  console.log(result); 
  // {
  // added: [ [ 'd', 'd1' ] ],
  // removed: [ [ 'b', 'b1' ] ],
  // edited: [ [ 'c/c1', 'c1', 'c222' ] ]
  // }
 * 
 * ```
 */
export const diff = (oldStruct, newStruct) => {
  const delta = {
    added: [],
    removed: [],
    edited: [],
  };
  const oldStructPaths = getStructPaths(oldStruct);
  const newStructPaths = getStructPaths(newStruct);

  delta.removed = getPathsDiff(oldStructPaths, newStructPaths);
  delta.added = getPathsDiff(newStructPaths, oldStructPaths);
  delta.edited = getEditedPaths(oldStructPaths, newStructPaths);

  return delta;
};
